@@ -24,6 +24,11 @@ STATES = {
     'readonly': Not(Equal(Eval('state'), 'draft')),
 }
 DEPENDS = ['state']
+FORECAST_STATES = [
+    ('draft', 'Draft'),
+    ('done', 'Done'),
+    ('cancel', 'Cancel'),
+    ]
 
 
 class Forecast(Workflow, ModelSQL, ModelView):
@@ -54,11 +59,8 @@ class Forecast(Workflow, ModelSQL, ModelView):
                 Bool(Eval('lines', [0]))),
             },
         depends=['state'])
-    state = fields.Selection([
-        ('draft', 'Draft'),
-        ('done', 'Done'),
-        ('cancel', 'Cancel'),
-        ], 'State', readonly=True, select=True)
+    state = fields.Selection(
+        FORECAST_STATES, 'State', readonly=True, select=True)
     active = fields.Function(fields.Boolean('Active'),
         'get_active', searcher='search_active')
 
@@ -194,10 +196,13 @@ class Forecast(Workflow, ModelSQL, ModelView):
             forecast.check_date_overlap()
 
     def check_date_overlap(self):
-        cursor = Transaction().connection.cursor()
         if self.state != 'done':
             return
+        transaction = Transaction()
+        connection = transaction.connection
+        transaction.database.lock(connection, self._table)
         forcast = self.__table__()
+        cursor = connection.cursor()
         cursor.execute(*forcast.select(forcast.id,
                 where=(((forcast.from_date <= self.from_date)
                         & (forcast.to_date >= self.from_date))
@@ -252,10 +257,15 @@ class Forecast(Workflow, ModelSQL, ModelView):
     @staticmethod
     def create_moves(forecasts):
         'Create stock moves for the forecast ids'
+        pool = Pool()
+        Line = pool.get('stock.forecast.line')
+        to_save = []
         for forecast in forecasts:
             if forecast.state == 'done':
                 for line in forecast.lines:
-                    line.create_moves()
+                    line.moves += tuple(line.get_moves())
+                    to_save.append(line)
+        Line.save(to_save)
 
     @staticmethod
     def delete_moves(forecasts):
@@ -288,11 +298,17 @@ class ForecastLine(ModelSQL, ModelView):
     'Stock Forecast Line'
     __name__ = 'stock.forecast.line'
     _rec_name = 'product'
+    _states = {
+        'readonly': Eval('forecast_state') != 'draft',
+        }
+    _depends = ['forecast_state']
+
     product = fields.Many2One('product.product', 'Product', required=True,
         domain=[
             ('type', '=', 'goods'),
             ('consumable', '=', False),
-            ])
+            ],
+        states=_states, depends=_depends)
     product_uom_category = fields.Function(
         fields.Many2One('product.uom.category', 'Product Uom Category'),
         'on_change_with_product_uom_category')
@@ -301,22 +317,34 @@ class ForecastLine(ModelSQL, ModelView):
             If(Bool(Eval('product_uom_category')),
                 ('category', '=', Eval('product_uom_category')),
                 ('category', '!=', -1)),
-            ], depends=['product', 'product_uom_category'])
+            ],
+        states=_states,
+        depends=['product', 'product_uom_category'] + _depends)
     unit_digits = fields.Function(fields.Integer('Unit Digits'),
             'get_unit_digits')
     quantity = fields.Float('Quantity', digits=(16, Eval('unit_digits', 2)),
-        required=True, depends=['unit_digits'])
+        required=True, states=_states, depends=['unit_digits'] + _depends)
     minimal_quantity = fields.Float('Minimal Qty',
         digits=(16, Eval('unit_digits', 2)), required=True,
-        depends=['unit_digits'])
+        states=_states, depends=['unit_digits'] + _depends)
     moves = fields.Many2Many('stock.forecast.line-stock.move',
         'line', 'move', 'Moves', readonly=True)
     forecast = fields.Many2One(
-        'stock.forecast', 'Forecast', required=True, ondelete='CASCADE')
+        'stock.forecast', 'Forecast', required=True, ondelete='CASCADE',
+        states={
+            'readonly': ((Eval('forecast_state') != 'draft')
+                & Bool(Eval('forecast'))),
+            },
+        depends=['forecast_state'])
+    forecast_state = fields.Function(
+        fields.Selection(FORECAST_STATES, 'Forecast State'),
+        'on_change_with_forecast_state')
     quantity_executed = fields.Function(fields.Float('Quantity Executed',
             digits=(16, Eval('unit_digits', 2)), depends=['unit_digits']),
         'get_quantity_executed')
 
+    del _states, _depends
+
     @classmethod
     def __setup__(cls):
         super(ForecastLine, cls).__setup__()
@@ -360,6 +388,11 @@ class ForecastLine(ModelSQL, ModelView):
     def get_unit_digits(self, name):
         return self.product.default_uom.digits
 
+    @fields.depends('forecast', '_parent_forecast.state')
+    def on_change_with_forecast_state(self, name=None):
+        if self.forecast:
+            return self.forecast.state
+
     @classmethod
     def get_quantity_executed(cls, lines, name):
         cursor = Transaction().connection.cursor()
@@ -417,8 +450,8 @@ class ForecastLine(ModelSQL, ModelView):
         default['moves'] = None
         return super(ForecastLine, cls).copy(lines, default=default)
 
-    def create_moves(self):
-        'Create stock moves for the forecast line'
+    def get_moves(self):
+        'Get stock moves for the forecast line'
         pool = Pool()
         Move = pool.get('stock.move')
         Uom = pool.get('product.uom')
@@ -445,27 +478,23 @@ class ForecastLine(ModelSQL, ModelView):
             unit_price = Uom.compute_price(self.product.default_uom,
                 unit_price, self.uom)
 
-        to_create = []
+        moves = []
         for day, qty in distribution.iteritems():
             if qty == 0.0:
                 continue
-            to_create.append({
-                    'from_location': (
-                        self.forecast.warehouse.storage_location.id),
-                    'to_location': self.forecast.destination.id,
-                    'product': self.product.id,
-                    'uom': self.uom.id,
-                    'quantity': qty * self.minimal_quantity,
-                    'planned_date': (self.forecast.from_date
-                        + datetime.timedelta(day)),
-                    'company': self.forecast.company.id,
-                    'currency': self.forecast.company.currency.id,
-                    'unit_price': unit_price,
-                    })
-        moves = []
-        if to_create:
-            moves = Move.create(to_create)
-        self.write([self], {'moves': [('add', [m.id for m in moves])]})
+            move = Move()
+            move.from_location = self.forecast.warehouse.storage_location
+            move.to_location = self.forecast.destination
+            move.product = self.product
+            move.uom = self.uom
+            move.quantity = qty * self.minimal_quantity
+            move.planned_date = (self.forecast.from_date
+                + datetime.timedelta(day))
+            move.company = self.forecast.company
+            move.currency = self.forecast.company.currency
+            move.unit_price = unit_price
+            moves.append(move)
+        return moves
 
     @classmethod
     def delete_moves(cls, lines):
@@ -593,12 +622,14 @@ class ForecastComplete(Wizard):
 
     def transition_complete(self):
         pool = Pool()
+        Forecast = pool.get('stock.forecast')
         ForecastLine = pool.get('stock.forecast.line')
         Product = pool.get('product.product')
 
+        forecast = Forecast(Transaction().context['active_id'])
         prod2line = {}
         forecast_lines = ForecastLine.search([
-                ('forecast', '=', Transaction().context['active_id']),
+                ('forecast', '=', forecast.id),
                 ])
         for forecast_line in forecast_lines:
             prod2line[forecast_line.product.id] = forecast_line
@@ -614,7 +645,7 @@ class ForecastComplete(Wizard):
         else:
             products = None
 
-        to_create = []
+        to_save = []
         for key, qty in pbl.iteritems():
             _, product = key
             if products and product not in products:
@@ -622,21 +653,15 @@ class ForecastComplete(Wizard):
             if -qty <= 0:
                 continue
             if product in prod2line:
-                ForecastLine.write([prod2line[product]], {
-                        'product': product,
-                        'quantity': -qty,
-                        'uom': prod2uom[product],
-                        'forecast': Transaction().context['active_id'],
-                        'minimal_quantity': min(1, -qty),
-                        })
+                line = prod2line[product]
             else:
-                to_create.append({
-                        'product': product,
-                        'quantity': -qty,
-                        'uom': prod2uom[product],
-                        'forecast': Transaction().context['active_id'],
-                        'minimal_quantity': min(1, -qty),
-                        })
-        if to_create:
-            ForecastLine.create(to_create)
+                line = ForecastLine()
+            line.product = product
+            line.quantity = -qty
+            line.uom = prod2uom[product]
+            line.forecast = forecast
+            line.minimal_quantity = min(1, -qty)
+            to_save.append(line)
+
+        ForecastLine.save(to_save)
         return 'end'
